@@ -58,8 +58,14 @@
 #'              A value closer to 1 indicates greater internal consistency.
 #'            }
 #'            \item{\code{cronb()}}{
-#'              The Cronbach's Alpha value for \code{x}.
-#'              A value closer to 1 indicates greater internal consistency.
+#'              The Cronbach's Alpha value for \code{x}. A value closer to 1
+#'              indicates greater internal consistency, where usually following
+#'              rule of thumb is applied to interprete the results:
+#'              \ifelse{html}{\out{&alpha;}}{\eqn{\alpha}{alpha}} < 0.5 is unacceptable,
+#'              0.5 < \ifelse{html}{\out{&alpha;}}{\eqn{\alpha}{alpha}} < 0.6 is poor,
+#'              0.6 < \ifelse{html}{\out{&alpha;}}{\eqn{\alpha}{alpha}} < 0.7 is questionable,
+#'              0.7 < \ifelse{html}{\out{&alpha;}}{\eqn{\alpha}{alpha}} < 0.8 is acceptable,
+#'              and everything > 0.8 is good or excellent.
 #'            }
 #'            \item{\code{mic()}}{
 #'              This function calculates a mean inter-item-correlation, i.e.
@@ -79,7 +85,7 @@
 #'              \cite{(Piedmont 2014)}
 #'            }
 #'            \item{\code{difficulty()}}{
-#'              This function calculates the irem difficutly, which should
+#'              This function calculates the item difficutly, which should
 #'              range between 0.2 and 0.8. Lower values are a signal for
 #'              more difficult items, while higher values close to one
 #'              are a sign for easier items. The ideal value for item difficulty
@@ -147,7 +153,6 @@
 #'  }}
 #'
 #' @importFrom stats cor
-#' @importFrom tibble tibble
 #' @importFrom sjmisc std
 #' @export
 reliab_test <- function(x, scale.items = FALSE, digits = 3, out = c("txt", "viewer", "browser")) {
@@ -202,7 +207,7 @@ reliab_test <- function(x, scale.items = FALSE, digits = 3, out = c("txt", "view
     }
 
     # create return value
-    ret.df <- tibble::tibble(
+    ret.df <- data_frame(
       term = df.names,
       alpha.if.deleted = round(cronbachDeleted, digits),
       item.discr = round(totalCorr, digits)
